@@ -23,24 +23,22 @@ configparser::configparser() {
 
 configparser::~configparser() { }
 
-action_handler_status configparser::handle_action(const std::string& action, const std::vector<std::string>& params) {
+void configparser::handle_action(const std::string& action, const std::vector<std::string>& params) {
 	/*
 	 * configparser also acts as config_action_handler to implement to recursive "include" command.
 	 */
 	if (action == "include") {
 		if (params.size() < 1) {
-			return AHS_TOO_FEW_PARAMS;
+			throw confighandlerexception(AHS_TOO_FEW_PARAMS);
 		}
 
-		if (this->parse(utils::resolve_tilde(params[0])))
-			return AHS_OK;
-		else
-			return AHS_FILENOTFOUND;
-	}
-	return AHS_INVALID_COMMAND;
+		if (!this->parse(utils::resolve_tilde(params[0])))
+			throw confighandlerexception(AHS_FILENOTFOUND);
+	} else
+		throw confighandlerexception(AHS_INVALID_COMMAND);
 }
 
-bool configparser::parse(const std::string& filename) {
+bool configparser::parse(const std::string& filename, bool double_include) {
 	/*
 	 * this function parses a config file.
 	 *
@@ -53,8 +51,8 @@ bool configparser::parse(const std::string& filename) {
 	 *   - hand over the tokenize results to the config_action_handler
 	 *   - if an error happens, react accordingly.
 	 */
-	if (included_files.find(filename) != included_files.end()) {
-		GetLogger().log(LOG_WARN, "configparser::parse: file %s has already been included", filename.c_str());
+	if (!double_include && included_files.find(filename) != included_files.end()) {
+		LOG(LOG_WARN, "configparser::parse: file %s has already been included", filename.c_str());
 		return true;
 	}
 	included_files.insert(included_files.begin(), filename);
@@ -64,21 +62,22 @@ bool configparser::parse(const std::string& filename) {
 	std::string line;
 	getline(f,line);
 	if (!f.is_open()) {
-		GetLogger().log(LOG_WARN, "configparser::parse: file %s couldn't be opened", filename.c_str());
+		LOG(LOG_WARN, "configparser::parse: file %s couldn't be opened", filename.c_str());
 		return false;
 	}
 	while (f.is_open() && !f.eof()) {
-		GetLogger().log(LOG_DEBUG,"configparser::parse: tokenizing %s",line.c_str());
+		LOG(LOG_DEBUG,"configparser::parse: tokenizing %s",line.c_str());
 		std::vector<std::string> tokens = utils::tokenize_quoted(line);
 		if (tokens.size() > 0) {
 			std::string cmd = tokens[0];
 			config_action_handler * handler = action_handlers[cmd];
 			if (handler) {
 				tokens.erase(tokens.begin()); // delete first element
-				action_handler_status status = handler->handle_action(cmd,tokens);
-				if (status != AHS_OK) {
-					const char * errmsg = get_errmsg(status);
-					throw configexception(utils::strprintf(_("Error while processing command `%s' (%s line %u): %s"), line.c_str(), filename.c_str(), linecounter, errmsg));
+				try {
+					evaluate_backticks(tokens);
+					handler->handle_action(cmd,tokens);
+				} catch (const confighandlerexception& e) {
+					throw configexception(utils::strprintf(_("Error while processing command `%s' (%s line %u): %s"), line.c_str(), filename.c_str(), linecounter, e.what()));
 				}
 			} else {
 				throw configexception(utils::strprintf(_("unknown command `%s'"), cmd.c_str()));
@@ -91,28 +90,34 @@ bool configparser::parse(const std::string& filename) {
 }
 
 void configparser::register_handler(const std::string& cmd, config_action_handler * handler) {
-	GetLogger().log(LOG_DEBUG,"configparser::register_handler: cmd = %s handler = %p", cmd.c_str(), handler);
 	action_handlers[cmd] = handler;
 }
 
 void configparser::unregister_handler(const std::string& cmd) {
-	GetLogger().log(LOG_DEBUG,"configparser::unregister_handler: cmd = %s", cmd.c_str());
 	action_handlers[cmd] = 0;
 }
 
-const char * configparser::get_errmsg(action_handler_status status) {
-	switch (status) {
-		case AHS_INVALID_PARAMS:
-			return _("invalid parameters.");
-		case AHS_TOO_FEW_PARAMS:
-			return _("too few parameters.");
-		case AHS_INVALID_COMMAND:
-			return _("unknown command (bug).");
-		case AHS_FILENOTFOUND:
-			return _("file couldn't be opened.");
-		default:
-			return _("unknown error (bug).");
+void configparser::evaluate_backticks(std::vector<std::string>& tokens) {
+	for (std::vector<std::string>::iterator it=tokens.begin();it!=tokens.end();it++) {
+		*it = evaluate_backticks(*it);
 	}
 }
 
+std::string configparser::evaluate_backticks(std::string token) {
+	std::string::size_type pos1 = token.find_first_of("`", 0);
+	std::string::size_type pos2 = 0;
+	while (pos1 != std::string::npos && pos2 != std::string::npos) {
+		pos2 = token.find_first_of("`", pos1+1);
+		if (pos2 != std::string::npos) {
+			std::string cmd = token.substr(pos1+1, pos2-pos1-1);
+			token.erase(pos1, pos2-pos1+1);
+			std::string result = utils::get_command_output(cmd);
+			utils::trim_end(result);
+			token.insert(pos1, result);
+			pos1 = token.find_first_of("`", pos1+result.length()+1);
+		}
+	}
+	return token;
+}
+
 }
