@@ -261,7 +261,7 @@ std::string utils::convert_text(const std::string& text, const std::string& toco
  * of all the Unix-like systems around there, only Linux/glibc seems to 
  * come with a SuSv3-conforming iconv implementation.
  */
-#if !(__linux) && !defined(__GLIBC__)
+#if !(__linux) && !defined(__GLIBC__) && !defined(__APPLE__)
 	const char * inbufp;
 #else
 	char * inbufp;
@@ -608,6 +608,21 @@ size_t utils::strwidth(const std::string& str) {
 	return width; // exact width
 }
 
+size_t utils::strwidth_stfl(const std::string& str) {
+	size_t reduce_count = 0;
+	size_t len = str.length();
+	if (len > 1) {
+		for (size_t idx=0;idx<len-1;++idx) {
+			if (str[idx] == '<' && str[idx+1] != '>') {
+				reduce_count += 3;
+				idx += 3;
+			}
+		}
+	}
+
+	return strwidth(str) - reduce_count;
+}
+
 std::string utils::join(const std::vector<std::string>& strings, const std::string& separator) {
 	std::string result;
 
@@ -715,7 +730,6 @@ void utils::set_common_curl_options(CURL * handle, configcontainer * cfg) {
 	std::string useragent; 
 	unsigned int dl_timeout = 0;
 
-
 	if (cfg) {
 		if (cfg->get_configvalue_as_bool("use-proxy")) {
 			proxy = cfg->get_configvalue("proxy");
@@ -747,6 +761,34 @@ void utils::set_common_curl_options(CURL * handle, configcontainer * cfg) {
 	curl_easy_setopt(handle, CURLOPT_FAILONERROR, 1);
 }
 
+std::string utils::get_content(xmlNode * node) {
+	std::string retval;
+	if (node) {
+		xmlChar * content = xmlNodeGetContent(node);
+		if (content) {
+			retval = (const char *)content;
+			xmlFree(content);
+		}
+	}
+	return retval;
+}
+
+std::string utils::get_prop(xmlNode * node, const char * prop, const char * ns) {
+	std::string retval;
+	if (node) {
+		xmlChar * value;
+		if (ns)
+			value = xmlGetProp(node, (xmlChar *)prop);
+		else
+			value = xmlGetNsProp(node, (xmlChar *)prop, (xmlChar *)ns);
+		if (value) {
+			retval = (const char*)value;
+			xmlFree(value);
+		}
+	}
+	return retval;
+}
+
 curl_proxytype utils::get_proxy_type(const std::string& type) {
 	if (type == "http")
 		return CURLPROXY_HTTP;
@@ -757,8 +799,26 @@ curl_proxytype utils::get_proxy_type(const std::string& type) {
 	if (type == "socks4a")
 		return CURLPROXY_SOCKS4A;
 
-	LOG(LOG_USERERROR, "you configured an invalid proxy type: %s", type.c_str());
+	if (type != "") {
+		LOG(LOG_USERERROR, "you configured an invalid proxy type: %s", type.c_str());
+	}
 	return CURLPROXY_HTTP;
 }
 
+std::string utils::escape_url(const std::string& url) {
+	return replace_all(replace_all(url,"?","%3F"), "&", "%26");
+}
+
+std::string utils::unescape_url(const std::string& url) {
+	return replace_all(replace_all(url,"%3F","?"), "%26", "&");
+}
+
+std::wstring utils::clean_nonprintable_characters(std::wstring text) {
+	for (size_t idx=0;idx<text.size();++idx) {
+		if (!iswprint(text[idx]))
+			text[idx] = L'\uFFFD';
+	}
+	return text;
+}
+
 }
